# -*- coding:utf-8 -*-
"""Database and Data models for att-bill-splitter."""

from peewee import *
from attbillsplitter.utils import DATABASE_PATH

db = SqliteDatabase(DATABASE_PATH)


class BaseModel(Model):
    class Meta:
        database = db


class User(BaseModel):
    name = CharField()
    number = CharField()
    created_at = DateTimeField(constraints=[SQL("DEFAULT (datetime('now'))")])

    class Meta:
        indexes = (
            (('name', 'number'), True),
        )


class ChargeCategory(BaseModel):
    category = CharField(unique=True)
    text = CharField()
    created_at = DateTimeField(constraints=[SQL("DEFAULT (datetime('now'))")])


class ChargeType(BaseModel):
    type = CharField()
    text = CharField()
    charge_category = ForeignKeyField(ChargeCategory)
    created_at = DateTimeField(constraints=[SQL("DEFAULT (datetime('now'))")])

    class Meta:
        indexes = (
            (('type', 'charge_category'), True),
        )


class BillingCycle(BaseModel):
    name = CharField(unique=True)
    start_date = DateField(unique=True)
    end_date = DateField(unique=True)
    created_at = DateTimeField(constraints=[SQL("DEFAULT (datetime('now'))")])


class Charge(BaseModel):
    user = ForeignKeyField(User)
    charge_type = ForeignKeyField(ChargeType)
    billing_cycle = ForeignKeyField(BillingCycle)
    amount = FloatField()
    created_at = DateTimeField(constraints=[SQL("DEFAULT (datetime('now'))")])

    class Meta:
        indexes = (
            # create a unique on user, charge_type, billing_cycle
            (('user', 'charge_type', 'billing_cycle'), True),
        )


class MonthlyBill(BaseModel):
    user = ForeignKeyField(User, related_name='mb_user')
    billing_cycle = ForeignKeyField(BillingCycle,
                                    related_name='mb_billing_cycle')
    total = FloatField()
    created_at = DateTimeField(constraints=[SQL("DEFAULT (datetime('now'))")])



# -*- coding:utf-8 -*-
"""Utility methods"""

from __future__ import print_function, unicode_literals
from builtins import input
try:
    import configparser
except:
    import ConfigParser as configparser
import os
import sys
import warnings

CONFIG_PATH = os.path.expanduser('~/.attbillsplitter.conf')
PAGE_LOADING_WAIT_S = 10
DATABASE_PATH = 'att_bill.db'
LOG_PATH = 'notif_history.log'
warnings.simplefilter('ignore')


def initialize_twiolio():
    """Initialize twilio credentials from command line input and save in
    config file.

    :returns: None
    """
    number = input('Twilio Number (e.g. +11234567890): ')
    account_sid = input('Twilio Account SID: ')
    auth_token = input('Twilio Authentication Token: ')
    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)
    if config.remove_section('twilio'):
        print('\U00002B55  Old twilio credentials removed.')

    config.add_section('twilio')
    config.set('twilio', 'number', number)
    config.set('twilio', 'account_sid', account_sid)
    config.set('twilio', 'auth_token', auth_token)
    with open(CONFIG_PATH, 'w') as configfile:
        config.write(configfile)
    print('\U00002705  New twilio account added.')


def load_twilio_config():
    """Load twilio credentials. Prompt to initialize if not yet initialized.

    :returns: a tuple of twilio number, sid and auth token
    :rtype: tuple
    """
    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)
    # initialize twilio if not yet initialized
    if 'twilio' not in config.sections():
        initialize_twiolio()
        config.read(CONFIG_PATH)

    number = config.get('twilio', 'number')
    account_sid = config.get('twilio', 'account_sid')
    auth_token = config.get('twilio', 'auth_token')
    return (number, account_sid, auth_token)


def initialize_payment_msg():
    """Initialize payment message to be appended to the charging details
    before sending to users (generally a mesaage to tell users how to pay you).

    :returns: None
    """
    prompt_msg = ('You can enter a short message to put after the charge '
                  'details to send to your users. (For example, letting your '
                  'users know how to pay you)\n-> ')
    message = input(prompt_msg)
    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)
    if config.remove_section('message'):
        print('\U00002B55  Old payment message removed.')
    config.add_section('message')
    config.set('message', 'payment', message)
    with open(CONFIG_PATH, 'w') as configfile:
        config.write(configfile)
    print('\U00002705  New payment message saved.')


def load_payment_msg():
    """Load payment message. Prompt to initialize if not yet initialized.

    :returns: payment message cached in config file
    :rtype: str
    """
    config = configparser.ConfigParser()
    config.read(CONFIG_PATH)
    # initialize twilio if not yet initialized
    if ('message' not in config.sections() or
            'payment' not in [t for (t, _) in config.items('message')]):
        initialize_payment_msg()
        config.read(CONFIG_PATH)

    else:
        message = config.get('message', 'payment')
        prompt = ('\U00002753  Do you want to keep using the following '
                  'message: \n{}\n(y/n)? '.format(message))
        try:
            # python3
            reset = input(prompt)
        except UnicodeEncodeError:
            # python2
            reset = input(prompt.encode(sys.stdout.encoding))
        if reset in ('n', 'N', 'no', 'No', 'No'):
            initialize_payment_msg()
            config.read(CONFIG_PATH)
    return config.get('message', 'payment')



# -*- coding:utf-8 -*-
"""Entrypoints for att-bill-splitter:
    * split-bill
"""


def split_bill():
    """Parse AT&T bill and split wireless charges among users."""
    from attbillsplitter.main import run_split_bill
    run_split_bill()


def print_summary():
    """Print wireless monthly summary among users."""
    from attbillsplitter.services import run_print_summary
    run_print_summary()


def print_details():
    """Print wireless monthly details among users."""
    from attbillsplitter.services import run_print_details
    run_print_details()


def notify_users():
    """Print wireless monthly details among users."""
    from attbillsplitter.services import run_notify_users
    run_notify_users()


def init_twilio():
    """Initialize twilio credentials."""
    from attbillsplitter.utils import initialize_twiolio
    initialize_twiolio()


def init_payment_msg():
    """Initialize twilio credentials."""
    from attbillsplitter.utils import initialize_payment_msg
    initialize_payment_msg()



# -*- coding: utf-8 -*-
"""Main module"""

from __future__ import print_function, unicode_literals
import datetime as dt
import re
import click
import peewee as pw
import requests
from bs4 import BeautifulSoup, Tag
from slugify import slugify
# import fake_useragent
from attbillsplitter.errors import ParsingError
from attbillsplitter.models import (
    User, ChargeCategory, ChargeType, BillingCycle, Charge, MonthlyBill, db
)


CHROME_AGENT = ('Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 '
                '(KHTML, like Gecko) Chrome/28.0.1468.0 Safari/537.36')
# CHROME_AGENT = fake_useragent.UserAgent().chrome


def create_tables_if_not_exist():
    """Create tables in database if tables do not exist.

    Tables will be created for following models:
        - User
        - ChargeCategory
        - ChargeType
        - BillingCycle
        - Charge
        - MonthlyBill
    """
    db.connect()
    for model in (User, ChargeCategory, ChargeType, BillingCycle, Charge,
                  MonthlyBill):
        if not model.table_exists():
            model.create_table()


def get_start_end_date(bc_name):
    """Get start date and end date for a billing cycle name using regex.

    :param bc_name: name of billing cycle in format of
        'Mar 15 - Apr 14, 2016'
    :type bc_name: str
    :returns: a tuple of start date (datetime.date) and end date
    :rtype: tuple
    """
    m = re.search(r'(\w+ \d+) - (\w+ \d+), (\d{4})', bc_name)
    start_date_str = '{}, {}'.format(m.group(1), m.group(3))
    end_date_str = '{}, {}'.format(m.group(2), m.group(3))
    start_date = dt.datetime.strptime(start_date_str, '%b %d, %Y').date()
    end_date = dt.datetime.strptime(end_date_str, '%b %d, %Y').date()
    return (start_date, end_date)


def aggregate_wireless_monthly(bc):
    """Aggregate wireless charges among all lines.

    :param bc: billing_cycle object
    :type bc: BillingCycle
    :returns: None
    """
    if MonthlyBill.select().where(MonthlyBill.billing_cycle == bc).exists():
        print('\U000026A0  Charges already aggregated for {}'.format(bc.name))
        return

    query = (
        User
        .select(User,
                pw.fn.SUM(Charge.amount).alias('total'))
        .join(Charge)
        .join(BillingCycle)
        .switch(Charge)
        .join(ChargeType)
        .join(ChargeCategory)
        .where(BillingCycle.id == bc.id,
               ChargeCategory.category == 'wireless')
        .group_by(User, BillingCycle)
        .naive()
    )
    results = query.execute()
    for user in results:
        MonthlyBill.create(user=user, billing_cycle=bc, total=user.total)


class AttBillSplitter(object):
    """Parse AT&T bill and split wireless charges among users.

    Currently tested on AT&T account with U-verse TV, Internet and Mobile
    Share Value Plan (for wireless).
    """

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.session = requests.session()
        headers = {'User-Agent': CHROME_AGENT}
        self.session.headers.update(headers)

    def login(self):
        """Login to your AT&T online account.

        :returns: status of the login (True or False)
        :rtype: bool
        """
        print('\U000025B6  Login started...')
        login_url = (
            'https://myattdx05.att.com/commonLogin/igate_wam/multiLogin.do'
        )
        # obtain session cookies needed to login
        self.session.get(login_url)
        # soup = BeautifulSoup(pre_login.text, 'html.parser')
        # hidden_inputs = soup.find_all('input', type='hidden')
        # form = {
        #     x.get('name'): x.get('value')
        #     for x in hidden_inputs if x.get('value') and x.get('name')
        # }

        # this information is in a comment tag and is very difficult to parse
        # hard code this for now
        form = {
            'source': 'MYATT',
            'flow_ind': 'LGN',
            'isSlidLogin': 'true',
            'vhname': 'www.att.com',
            'urlParameters': ('fromdlom=true&reportActionEvent=A_LGN_LOGIN_SUB'
                              '&loginSource=olam'),
            'myATTIntercept': 'false',
            'persist': 'Y',
            'rootPath': '/olam/English'
        }
        form.update({'userid': self.username, 'password': self.password})
        login_submit = self.session.post(login_url, data=form)
        # open('test.html', 'w').write(login_submit.text.encode('utf-8'))
        if ('Your total balance is:' in login_submit.text or
                'Your total credit balance is' in login_submit.text):
            print('\U00002705  Login succeeded.')
            return True

        else:
            if 'promo' in login_submit.url.lower():
                print ('\U0001F534  Popup window detected during login. '
                       'Please login you account in a browser and click '
                       'through. Log out your account before you retry. '
                       '(Sometimes you might have to do this multiple times.')
                return False

            print('\U0001F6AB  Login failed. Please check your username and '
                  'password and retry. Or something unexpected happened.')
            return False

    def get_history_bills(self):
        """Get history bills.

        :yields: tuple of billing_cycle name and link to the bill
        """

        # this request will add some cookie
        self.session.get(
            'https://www.att.com/olam/passthroughAction.myworld',
            params={'actionType': 'ViewBillHistory'}
        )
        # get account number
        an_req = self.session.get(
            'https://www.att.com/olam/acctInfoView.myworld',
            params={'actionEvent': 'displayProfileInformation'}
        )
        an_soup = BeautifulSoup(an_req.text, 'html.parser')
        act_num_tag = an_soup.find('span', class_='account-number')
        m = re.search(r'.?(\d+).?', act_num_tag.text, re.DOTALL)
        if not m:
            raise ParsingError('Account number not found!')
        act_num = m.group(1)

        # now we can get billing history
        bh_req = self.session.get(
            'https://www.att.com/olam/billingPaymentHistoryAction.myworld',
            params={'action': 'ViewBillHistory'}
        )
        bh_req.raise_for_status()
        bh_soup = BeautifulSoup(bh_req.text, 'html.parser')
        bc_tags = bh_soup.find_all('td', headers=['bill_period'])
        bill_link_template = (
            'https://www.att.com/olam/billPrintPreview.myworld?'
            'fromPage=history&billStatementID={}|{}|T06|V'
        )
        for tag in bc_tags:
            bc_name = tag.contents[0]
            end_date_name = bc_name.split(' - ')[1]
            end_date = dt.datetime.strptime(end_date_name, '%b %d, %Y')
            end_date_str = end_date.strftime('%Y%m%d')
            bill_link = bill_link_template.format(end_date_str, act_num)
            yield (bc_name, bill_link)

    def parse_user_info(self, bill_html):
        """Parse the bill to find name and number for each line and create
        users. Account holder should be the first entry.

        :returns: list of user objects
        :rtype: list
        :returns: None
        """
        users = []
        soup = BeautifulSoup(bill_html, 'html.parser')
        number_tags = soup.find_all('div', string=re.compile('Total for'))
        for num_tag in number_tags:
            number = num_tag.text.lstrip('Total for')
            # get name for number
            name_tag = soup.find('div', class_='accRow bold MarTop10',
                                 string=re.compile('{}'.format(number)))
            name = name_tag.text.rstrip(' {}'.format(number))
            user, _ = User.get_or_create(name=name, number=number)
            users.append(user)
        return users

    def split_bill(self, bc_name, bill_link):
        """Parse bill and split wireless charges among users.

        Currently not parsing U-Verse charges.
        :param bc_name: billing cycle name
        :type bc_name: str
        :param bill_link: url to bill
        :type bc_name: str
        :returns: None
        """
        bill_req = self.session.get(bill_link)
        bill_html = bill_req.text
        if 'Account Details' not in bill_html:
            raise ParsingError('Failed to retrieve billing page')

        soup = BeautifulSoup(bill_html, 'html.parser')
        start_date, end_date = get_start_end_date(bc_name)
        billing_cycle = BillingCycle.create(name=bc_name,
                                            start_date=start_date,
                                            end_date=end_date)

        # parse user name and number
        users = self.parse_user_info(bill_html)
        if not users:
            return

        account_holder = users[0]
        # --------------------------------------------------------------------
        # Wireless
        # --------------------------------------------------------------------
        wireless_charge_category, _ = ChargeCategory.get_or_create(
            category='wireless',
            text='Wireless'
        )
        charged_users = users[:1]
        name, number = account_holder.name, account_holder.number
        offset = 0.0
        # charge section starts with user name followed by his/her number
        target = soup.find('div',
                           string=re.compile('{} {}'.format(name, number)))
        for tag in target.parent.next_siblings:
            # all charge data are in divs
            if not isinstance(tag, Tag) or tag.name != 'div':
                continue

            # charge section ends with Total for number
            if 'Total for {}'.format(number) in tag.text:
                break

            # each charge type has 'accSummary' as one of its css classes
            if 'accSummary' in tag.get('class', []):
                charge_type_text = tag.find('div').text.strip('\n\t')
                if charge_type_text.startswith('Monthly Charges'):
                    charge_type_text = 'Monthly Charges'
                    # account monthly fee will be shared by all users
                    w_act_m = float(
                        re.search(r'\$([0-9.]+)', tag.text).group(1)
                    )
                    # national discount is applied to account monthly fee
                    m = re.search(
                        r'National Account Discount.*?\$([0-9.]+)',
                        tag.text, re.DOTALL
                    )
                    w_act_m_disc = float(m.group(1)) if m else 0.0
                    # this non-zero offset will be used to adjust account
                    # holder's total monthly charge
                    offset = w_act_m - w_act_m_disc

                m = re.search(
                    r'Total {}.*?\$([0-9.]+)'.format(charge_type_text),
                    tag.text,
                    flags=re.DOTALL
                )
                charge_total = float(m.group(1)) - offset
                # save data to db
                charge_type_name = slugify(charge_type_text)
                # ChargeType
                charge_type, _ = ChargeType.get_or_create(
                    type=charge_type_name,
                    text=charge_type_text,
                    charge_category=wireless_charge_category
                )
                # Charge
                new_charge = Charge(
                    user=account_holder,
                    charge_type=charge_type,
                    billing_cycle=billing_cycle,
                    amount=charge_total
                )
                new_charge.save()
                offset = 0.0

        # iterate regular users
        for user in users[1:]:
            charge_total = 0.0
            name, number = user.name, user.number
            # charge section starts with user name followed by his/her number
            target = soup.find('div',
                               string=re.compile('{} {}'.format(name, number)))
            for tag in target.parent.next_siblings:
                # all charge data are in divs
                if not isinstance(tag, Tag) or tag.name != 'div':
                    continue

                # charge section ends with Total for number
                if 'Total for {}'.format(number) in tag.text:
                    break

                # each charge type has 'accSummary' as one of its css classes
                if 'accSummary' in tag.get('class', []):
                    charge_type_text = tag.find('div').text.strip('\n\t')
                    if charge_type_text.startswith('Monthly Charges'):
                        charge_type_text = 'Monthly Charges'

                    m = re.search(
                        r'Total {}.*?\$([0-9.]+)'.format(charge_type_text),
                        tag.text,
                        flags=re.DOTALL
                    )
                    charge_total = float(m.group(1))
                    # save data to db
                    charge_type_name = slugify(charge_type_text)
                    # ChargeType
                    charge_type, _ = ChargeType.get_or_create(
                        type=charge_type_name,
                        text=charge_type_text,
                        charge_category=wireless_charge_category
                    )
                    # Charge
                    new_charge = Charge(
                        user=user,
                        charge_type=charge_type,
                        billing_cycle=billing_cycle,
                        amount=charge_total
                    )
                    new_charge.save()
            if charge_total > 0:
                charged_users.append(user)

        # update share of account monthly charges for each user
        # also calculate total wireless charges (for verification later)
        act_m_share = (w_act_m - w_act_m_disc) / len(charged_users)
        wireless_total = 0
        for user in charged_users:
            # ChargeType
            charge_type, _ = ChargeType.get_or_create(
                type='wireless-acount-monthly-charges-share',
                text='Account Monthly Charges Share',
                charge_category=wireless_charge_category
            )
            # Charge
            new_charge = Charge(
                user=user,
                charge_type=charge_type,
                billing_cycle=billing_cycle,
                amount=act_m_share
            )
            new_charge.save()
            user_total = Charge.select(
                pw.fn.Sum(Charge.amount).alias('total')
            ).join(
                ChargeType,
                on=Charge.charge_type_id == ChargeType.id
            ).where(
                (Charge.user == user),
                Charge.billing_cycle == billing_cycle,
                ChargeType.charge_category == wireless_charge_category
            )
            wireless_total += user_total[0].total

        # aggregate
        aggregate_wireless_monthly(billing_cycle)

    def run(self, lag, force):
        """
        :param lag: a list of lags indicating which bills to split
        :type lag: list
        :param force: a flag to force splitting the bill
        :type force: bool
        :returns: None
        """
        if not self.login():
            return

        for i, (bc_name, bill_link) in enumerate(self.get_history_bills()):
            # if lag is not empty, only split bills specified
            if lag and (i not in lag) and not force:
                continue

            # check if billing cycle already exist
            if BillingCycle.select().where(
                    BillingCycle.name == bc_name
            ):
                print('\U000026A0  Billing Cycle {} already '
                      'processed.'.format(bc_name))
                continue

            print('\U0001F3C3  Start splitting bill {}...'.format(bc_name))
            self.split_bill(bc_name, bill_link)
            print('\U0001F3C1  Finished splitting bill {}.'.format(bc_name))


@click.command()
@click.option('--lag', '-l', multiple=True, type=int)
@click.option('--force', '-f', default=False)
@click.option('--username', prompt='\U0001F464  AT&T Username')
@click.option('--password', prompt='\U0001F5DD  AT&T Password',
              hide_input=True)
def run_split_bill(username, password, lag, force):
    create_tables_if_not_exist()
    splitter = AttBillSplitter(username, password)
    splitter.run(lag, force)


if __name__ == '__main__':
    run_split_bill()



# -*- coding:utf-8 -*-
"""Service that aggregate monthly wireless charges for each line in account.
"""

from __future__ import print_function, unicode_literals
from builtins import input
import datetime as dt
import logging
import click
import peewee as pw
import warnings
import attbillsplitter.utils as utils
from twilio.rest import TwilioRestClient
from twilio.exceptions import TwilioException
from attbillsplitter.models import (
    User, ChargeCategory, ChargeType, BillingCycle, Charge, MonthlyBill, db
)

warnings.simplefilter('ignore')
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
ch = logging.FileHandler(utils.LOG_PATH)
ch.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)


def print_wireless_monthly_summary(month, year=None):
    """Get wireless monthly summary for all lines. Results will be printed
    to console.

    :param month: month (1 - 12) of the end date of billing cycle
    :type month: int
    :param year: year of the end of of billing cycle. Default to current year
    :type year: int
    :returns: None
    """
    # year value default to current year
    year = year or dt.date.today().year
    if not BillingCycle.select().where(
        db.extract_date('month', BillingCycle.end_date) == month,
        db.extract_date('year', BillingCycle.end_date) == year
    ).exists():
        print('No charge summary found for {}/{}. Please split the '
              'bill first'.format(year, month))
        return

    bc = BillingCycle.select().where(
        db.extract_date('month', BillingCycle.end_date) == month,
        db.extract_date('year', BillingCycle.end_date) == year
    ).get()
    print('\n--------------------------------------------------------------')
    print('    Charge Summary for Billing Cycle {}'.format(bc.name))
    print('--------------------------------------------------------------')
    query = (
        User
        .select(User.name,
                User.number,
                MonthlyBill.total)
        .join(MonthlyBill)
        .where(MonthlyBill.billing_cycle_id == bc.id)
        .naive()
    )
    wireless_total = 0
    for user in query.execute():
        print('    {:^18s} ({})      Total: {:.2f}'.format(
            user.name, user.number, user.total
        ))
        wireless_total += user.total
    print('--------------------------------------------------------------')
    print('{:>47}: {:.2f}\n'.format('Wireless Total', wireless_total))


def print_wireless_monthly_details(month, year=None):
    """Get wireless monthly details for all lines. Results will be printed
    to console.

    :param month: month (1 - 12) of the end date of billing cycle
    :type month: int
    :param year: year of the end of of billing cycle. Default to current year
    :type year: int
    :returns: None
    """
    # year value default to current year
    year = year or dt.date.today().year
    if not BillingCycle.select().where(
        db.extract_date('month', BillingCycle.end_date) == month,
        db.extract_date('year', BillingCycle.end_date) == year
    ).exists():
        print('No charge summary found for {}/{}. Please split the '
              'bill first'.format(year, month))
        return

    bc = BillingCycle.select().where(
        db.extract_date('month', BillingCycle.end_date) == month,
        db.extract_date('year', BillingCycle.end_date) == year
    ).get()
    query = (
        User
        .select(User.id,
                User.name,
                User.number,
                ChargeType.text.alias('charge_type'),
                pw.fn.SUM(Charge.amount).alias('total'))
        .join(Charge)
        .join(BillingCycle)
        .switch(Charge)
        .join(ChargeType)
        .join(ChargeCategory)
        .where(BillingCycle.id == bc.id,
               ChargeCategory.category == 'wireless')
        .group_by(User, BillingCycle, ChargeType)
        .order_by(User.id)
        .naive()
    )
    current_user_num = ''
    current_user_total = 0
    wireless_total = 0
    print('')
    for user in query.execute():
        if user.number != current_user_num:
            if current_user_total:
                print('      - {:40}   {:.2f}\n'.format('Total',
                                                        current_user_total))
                wireless_total += current_user_total
            current_user_num = user.number
            current_user_total = 0
            print('    {} ({})'.format(user.name, user.number))
        print('      - {:40}   {:.2f}'.format(user.charge_type, user.total))
        current_user_total += user.total
    if current_user_total:
        print('      - {:40}   {:.2f}\n'.format('Total', current_user_total))
        wireless_total += current_user_total
    print('{:>48}: {:.2f}\n'.format('Wireless Total', wireless_total))


def notify_users_monthly_details(message_client, payment_msg, month,
                                 year=None):
    """Calculate monthly charge details for users and notify them.

    :param message_client: a message client to send text message
    :type message_client: MessageClient
    :param payment_message: text appended to charge details so that your
        users know how to pay you.
    :param type: str
    :param month: month (1 - 12) of the end date of billing cycle
    :type month: int
    :param year: year of the end of of billing cycle. Default to current year
    :type year: int
    :returns: None
    """
    # year value default to current year
    year = year or dt.date.today().year
    if not BillingCycle.select().where(
        db.extract_date('month', BillingCycle.end_date) == month,
        db.extract_date('year', BillingCycle.end_date) == year
    ).exists():
        print('No charge summary found for {}/{}. Please split the '
              'bill first'.format(year, month))
        return

    bc = BillingCycle.select().where(
        db.extract_date('month', BillingCycle.end_date) == month,
        db.extract_date('year', BillingCycle.end_date) == year
    ).get()
    query = (
        User
        .select(User.id,
                User.name,
                User.number,
                ChargeType.text.alias('charge_type'),
                pw.fn.SUM(Charge.amount).alias('total'))
        .join(Charge)
        .join(BillingCycle)
        .switch(Charge)
        .join(ChargeType)
        .join(ChargeCategory)
        .where(BillingCycle.id == bc.id,
               ChargeCategory.category == 'wireless')
        .group_by(User, BillingCycle, ChargeType)
        .order_by(User.id)
        .naive()
    )
    current_user_num = -1
    current_user_total = 0
    messages = {}
    message = ''
    print('')
    for user in query.execute():
        if user.number != current_user_num:
            if current_user_total:
                message += '  - {:30} {:.2f} \U0001F911\n'.format(
                    'Total', current_user_total
                )
                messages[current_user_num] = message
            current_user_num = user.number
            current_user_total = 0
            message = ('Hi {} ({}),\nYour AT&T Wireless Charges '
                       'for {}:\n'.format(user.name, user.number, bc.name))
        message += '  - {:30} {:.2f}\n'.format(user.charge_type, user.total)
        current_user_total += user.total
    if current_user_total:
        message += '  - {:30} {:.2f} \U0001F911\n'.format('Total',
                                                          current_user_total)
        messages[current_user_num] = message
    # print message for user to confirm
    for num, msg in messages.items():
        print(num)
        print(msg)
        notify = input('Notify (y/n)? ')
        if notify in ('y', 'Y', 'yes', 'Yes', 'YES'):
            body = '{}\n{}'.format(msg, payment_msg)
            message_client.send_message(body=body, to=num)
            logger.info('%s charge details sent to %s, body:\n%s',
                        bc.name, num, msg)
            print('\U00002705  Message sent to {}\n'.format(num))


class MessageClient(object):
    """Twilio message client that sends text message to users."""
    def __init__(self):
        try:
            number, account_sid, auth_token = utils.load_twilio_config()
            self.number = number
            self.twilio_client = TwilioRestClient(account_sid, auth_token)
        except TwilioException:
            print('\U0001F6AB  Current twilio credentials invalid. '
                  'Please reset.')
            utils.initialize_twiolio()
            number, account_sid, auth_token = utils.load_twilio_config()
            self.number = number
            self.twilio_client = TwilioRestClient(account_sid, auth_token)

    def send_message(self, body, to):
        """Send message body from self.number to a phone number.

        :param body: message body to send
        :type body: str
        :param to: number to send message to (123-456-789)
        :type to: str
        :returns None
        """
        self.twilio_client.messages.create(body=body, to=to, from_=self.number)


@click.command()
@click.argument('month', type=int)
@click.option('-y', '--year', type=int)
def run_print_summary(month, year):
    """Print monthly charge summary for each user. MONTH refers to the month
    of the end date of the billing cycle. It should be an integer from 1 to
    12. You can also specify YEAR (in 4 digits). By default, YEAR is set to
    current calendar year.
    """
    print_wireless_monthly_summary(month, year)


@click.command()
@click.argument('month', type=int)
@click.option('-y', '--year', type=int)
def run_print_details(month, year):
    """Print monthly charge details for each user. MONTH refers to the month
    of the end date of the billing cycle. It should be an integer from 1 to
    12. You can also specify YEAR (in 4 digits). By default, YEAR is set to
    current calendar year.
    """
    print_wireless_monthly_details(month, year)


@click.command()
@click.argument('month', type=int)
@click.option('-y', '--year', type=int)
def run_notify_users(month, year):
    """Send monthly charge details to each user via SMS. For each user, you
    will first be shown his charge details, then you can decide whether you
    want to notify him/her. MONTH refers to the month of the end date of the
    billing cycle. It should be an integer from 1 to 12. You can also specify
    YEAR (in 4 digits). By default, YEAR is set to current calendar year.
    """
    mc = MessageClient()
    payment_msg = utils.load_payment_msg()
    notify_users_monthly_details(mc, payment_msg, month, year)



# -*- coding:utf-8 -*-
"""Setup for att-bill-splitter."""

from setuptools import find_packages, setup

setup(
    name='att-bill-splitter',
    # packages=['att-bill-splitter'],
    version='0.3',
    description='Parse AT&T bill and split wireless charges among users.',
    author='Brian Zhang',
    author_email='leapingzhang@gmail.com',
    url='https://github.com/brianzq/att-bill-splitter',
    # download_url=''
    install_requires=[
        'beautifulsoup4==4.5.1',
        'click>=6.6',
        'future>=0.16.0',
        # 'lxml==3.6.4',
        'peewee>=2.8.4',
        'python-slugify>=1.2.1',
        'requests==2.11.1',
        'twilio>=5.6.0',
        'Unidecode>=0.4.19',
    ],
    packages=find_packages(),
    extras_require={
        'testing': [
            'pytest>=2.9.2'
        ]
    },
    entry_points={
        'console_scripts': [
            'att-split-bill=attbillsplitter.entrypoints:split_bill',
            'att-print-summary=attbillsplitter.entrypoints:print_summary',
            'att-print-details=attbillsplitter.entrypoints:print_details',
            'att-notify-users=attbillsplitter.entrypoints:notify_users',
            'att-init-twilio=attbillsplitter.entrypoints:init_twilio',
            'att-init-payment-msg=attbillsplitter.entrypoints:init_payment_msg'
        ],
    },
    zip_safe=False
)



# -*- coding:utf-8 -*-
"""Test cases for att-bill-splitter."""

import datetime as dt
from attbillsplitter.main import get_start_end_date


def test_get_start_end_date():
    billing_cycle_name = 'Mar 15 - Apr 14, 2016'
    start_date = dt.date(2016, 3, 15)
    end_date = dt.date(2016, 4, 14)
    assert get_start_end_date(billing_cycle_name) == (start_date, end_date)



# -*- coding:utf-8 -*-
"""Exceptions used in main module."""

from peewee import IntegrityError


class BaseError(Exception):
    pass


class ConfigError(BaseError):
    pass


class UrlError(BaseError):
    pass


class LoginError(BaseError):
    pass


class ParsingError(BaseError):
    pass


class CalculationError(BaseError):
    pass


__all__ = ['ConfigError', 'UrlError', 'LoginError', 'ParsingError',
           'CalculationError', 'IntegrityError']
